%global with_debug 0

%if 0%{?with_debug}
%global _find_debuginfo_dwz_opts %{nil}
%global _dwz_low_mem_die_limit 0
%else
%global debug_package %{nil}
%endif

# binaries and unitfiles are currently called 'docker'
# to match with upstream supplied packages
%global origname docker
%global newname moby
%global service_name %{origname}

# moby
%global goipath_moby github.com/%{service_name}/%{service_name}
%global git_moby https://%{goipath_moby}
%global commit_moby e2f740de442bac52b280bc485a3ca5b31567d938
%global shortcommit_moby %(c=%{commit_moby}; echo ${c:0:7})

# cli
%global goipath_cli github.com/%{origname}/cli
%global git_cli https://%{goipath_cli}
%global commit_cli 485636f4b90ed5a91a1f403e65ffced469c641a
%global shortcommit_cli %(c=%{commit_cli}; echo ${c:0:7})

# tini
%global git_tini https://github.com/krallin/tini
%global commit_tini de40ad007797e0dcd8b7126f27bb87401d224240
%global shortcommit_tini %(c=%{commit_tini}; echo ${c:0:7})

Name: %{newname}-engine
Version: 20.10.14
Release: 1%{?dist}
Summary: The open-source application container engine
License: ASL 2.0
Source0: %{git_moby}/archive/v%{version}/moby-%{version}.tar.gz
Source1: %{git_cli}/archive/v%{version}/cli-%{version}.tar.gz
Source2: %{git_tini}/archive/%{commit_tini}/tini-%{shortcommit_tini}.tar.gz
Source3: %{service_name}.service
Source4: %{service_name}.sysconfig
# Seperate file containing virtual provides for bundled deps that's %%include'd in the specfile.
Source5: provides.spec.inc
URL: https://www.%{origname}.com

# BuildRequires: golang(github.com/docker/docker)
BuildRequires: golang(github.com/docker/libnetwork)
BuildRequires: pkgconfig(libbtrfsutil)
BuildRequires: pkgconfig(devmapper)
BuildRequires: golang
BuildRequires: go-rpm-macros
BuildRequires: go-md2man
BuildRequires: pkgconfig(libseccomp) >= 2.3.0
BuildRequires: make
BuildRequires: pkgconfig(audit)
BuildRequires: pkgconfig(systemd)
BuildRequires: firewalld-filesystem

# Build dependencies for tini
BuildRequires: cmake
BuildRequires: glibc-static

# required packages on install
Requires: container-selinux
Requires: iptables
Requires: systemd
Requires: tar
Requires: xz
Requires: pigz
Requires: runc
Requires: containerd

# Resolves: rhbz#1165615
Requires: device-mapper-libs >= 1.02.90-1

# Replace the old Docker packages
Obsoletes: %{origname} < 2:%{version}-%{release}
Obsoletes: %{origname}-latest < 2:%{version}-%{release}
Obsoletes: %{origname}-common < 2:%{version}-%{release}
Provides: %{origname} = %{version}-%{release}
Provides: %{origname}-latest = %{version}-%{release}

# conflicting packages
Conflicts: %{origname}
Conflicts: %{origname}-latest
Conflicts: %{origname}-common
Conflicts: %{origname}-io
Conflicts: %{origname}-engine-cs
Conflicts: %{origname}-ce
Conflicts: %{origname}-ce-cli
Conflicts: %{origname}-ee
Conflicts: podman-%{origname}

%include %{SOURCE5}

%description
Docker is an open source project to build, ship and run any application as a
lightweight container.

Docker containers are both hardware-agnostic and platform-agnostic. This means
they can run anywhere, from your laptop to the largest EC2 compute instance and
everything in between - and they don't require you to use a particular
language, framework or packaging system. That makes them great building blocks
for deploying and scaling web apps, databases, and backend services without
depending on a particular stack or provider.

%package fish-completion
Summary: Fish completion files for %{name}
Requires: %{name} = %{version}-%{release}
Requires: fish
Conflicts: %{service_name}-fish-completion
Obsoletes: %{service_name}-fish-completion < 2:%{version}-%{release}
Provides: %{service_name}-fish-completion = %{version}-%{release}

%description fish-completion
This package installs %{summary}.

%package zsh-completion
Summary: Zsh completion files for %{name}
Requires: %{name} = %{version}-%{release}
Requires: zsh
Conflicts: %{service_name}-zsh-completion
Obsoletes: %{service_name}-zsh-completion < 2:%{version}-%{release}
Provides: %{service_name}-zsh-completion = %{version}-%{release}

%description zsh-completion
This package installs %{summary}.

%package nano
Summary: GNU nano syntax highlighting files for Moby
Requires: %{name} = %{version}-%{release}
Requires: nano

%description nano
This package installs %{summary}.

%prep
%autosetup -p1 -n moby-%{version}

# untar cli
tar zxf %{SOURCE1}
# correct rpmlint errors for bash completion
sed -i '/env bash/d' cli-%{version}/contrib/completion/bash/docker

# untar tini
tar zxf %{SOURCE2}

%build
mkdir -p _build/bin

# build docker-proxy / libnetwork
(

        # Build binary using `golang-github-docker-libnetwork-devel`
        # (github.com/docker/libnetwork) installed in system GOPATH.
        export GOPATH="%{_datadir}/gocode"
        %gobuild -o _build/bin/%{service_name}-proxy github.com/%{service_name}/libnetwork/cmd/proxy
)

# build tini (installed as %%{service_name}-init)
(
        cd tini-%{commit_tini}
        %cmake .
        make tini-static -C "%{__cmake_builddir}"
)

%global buildtime %(date --utc --date="@${SOURCE_DATE_EPOCH:-$(date +%s)}" +"%Y-%m-%dT%H:%M:%SZ")
# build engine
(
        mkdir -p src/github.com/%{service_name}
        # Link source and vendored deps into local GOPATH.
        ln -fns ../../.. src/%{goipath_moby}
        # Build using source and vendored deps in local GOPATH.
        export GOPATH="${PWD}"
        export GO111MODULE=off
        export LDFLAGS="-w"
        export LDFLAGS+=" -X github.com/docker/docker/dockerversion.Version=%{version}"
        export LDFLAGS+=" -X github.com/docker/docker/dockerversion.GitCommit=%{shortcommit_moby}"
        export LDFLAGS+=" -X github.com/docker/docker/dockerversion.IAmStatic=false"
        export LDFLAGS+=" -X 'github.com/docker/docker/dockerversion.BuildTime=%{buildtime}'"
        export DOCKER_BUILDTAGS="seccomp selinux journald"
        %gobuild -o _build/bin/%{service_name}d %{goipath_moby}/cmd/%{service_name}d
        # VERSION=%%{version} DOCKER_GITCOMMIT=%%{shortcommit_moby} bash sh dynbinary
        # mv bundles/dynbinary-daemon/dockerd-%{version} _build/bin/dockerd
)

# build cli
(
        cd cli-%{version}
        mkdir -p src/github.com/%{service_name}
        # Link source and vendored deps into local GOPATH.
        ln -fns ../../.. src/%{goipath_cli}
        # export DISABLE_WARN_OUTSIDE_CONTAINER=1
        # Build using source and vendored deps in local GOPATH.
        export GOPATH="${PWD}"
        export GO111MODULE=off
        export LDFLAGS="\
            -w \
            -X \"github.com/docker/cli/cli/version.GitCommit=%{shortcommit_cli}\" \
            -X \"github.com/docker/cli/cli/version.BuildTime=%{buildtime}\" \
            -X \"github.com/docker/cli/cli/version.Version=%{version}\" \
    "
        %gobuild -o ../_build/bin/%{service_name} %{goipath_cli}/cmd/%{service_name}
        # make VERSION=%%{version} GITCOMMIT=%%{shortcommit_cli} dynbinary
        man/md2man-all.sh
)

%install
# install binary
install -Dpm 755 _build/bin/%{service_name} _build/bin/%{service_name}d -t %{buildroot}%{_bindir}/

# install proxy
install -Dpm 755 _build/bin/%{service_name}-proxy -t %{buildroot}%{_libexecdir}/%{service_name}/

# install tini
install -Dpm 755 tini-%{commit_tini}/%{__cmake_builddir}/tini-static %{buildroot}%{_libexecdir}/%{service_name}/%{service_name}-init

# install udev rules
install -Dpm 644 contrib/udev/80-%{service_name}.rules -t %{buildroot}%{_usr}/lib/udev/rules.d/

# add init scripts
install -Dpm 644 %{SOURCE3} contrib/init/systemd/docker.socket -t %{buildroot}%{_unitdir}/

# for additional args
install -Dpm 644 %{SOURCE4} %{buildroot}%{_sysconfdir}/sysconfig/%{service_name}

# add bash, zsh, and fish completions
install -Dpm 644 cli-%{version}/contrib/completion/bash/%{service_name} -t %{buildroot}%{_datadir}/bash-completion/completions/
install -Dpm 644 cli-%{version}/contrib/completion/zsh/_%{service_name} -t %{buildroot}%{_datadir}/zsh/site-functions/
install -Dpm 644 cli-%{version}/contrib/completion/fish/%{service_name}.fish -t %{buildroot}%{_datadir}/fish/vendor_completions.d/

# install manpages
install -Dpm 644 cli-%{version}/man/man1/*.1 -t %{buildroot}%{_mandir}/man1/
install -Dpm 644 cli-%{version}/man/man5/*.5 -t %{buildroot}%{_mandir}/man5/
install -Dpm 644 cli-%{version}/man/man8/*.8 -t %{buildroot}%{_mandir}/man8/

# add nano files
install -Dpm 644 contrib/syntax/nano/Dockerfile.nanorc -t %{buildroot}%{_datadir}/nano/

for cli_file in LICENSE MAINTAINERS NOTICE README.md; do
    cp "cli-%{version}/$cli_file" "cli-$cli_file"
done

%pre
getent group %{service_name} >/dev/null || groupadd -r %{service_name} || :

%post
%systemd_post %{service_name}.service %{service_name}.socket
%firewalld_reload

%preun
%systemd_preun %{service_name}.service %{service_name}.socket

%postun
%systemd_postun_with_restart %{service_name}.service

%files
%license LICENSE cli-LICENSE
%doc AUTHORS CHANGELOG.md CONTRIBUTING.md MAINTAINERS NOTICE README.md
%doc cli-MAINTAINERS cli-NOTICE cli-README.md
%config(noreplace) %{_sysconfdir}/sysconfig/%{service_name}
%{_bindir}/%{service_name}
%{_bindir}/%{service_name}d
%dir %{_libexecdir}/%{service_name}/
%{_libexecdir}/%{service_name}/%{service_name}-proxy
%{_libexecdir}/%{service_name}/%{service_name}-init
%{_usr}/lib/udev/rules.d/80-%{service_name}.rules
%{_unitdir}/%{service_name}.service
%{_unitdir}/%{service_name}.socket
%{_datadir}/bash-completion/completions/%{service_name}
%{_mandir}/man1/*
%{_mandir}/man5/*
%{_mandir}/man8/*

%files zsh-completion
%{_datadir}/zsh/site-functions/_%{service_name}

%files fish-completion
%{_datadir}/fish/vendor_completions.d/%{service_name}.fish

%files nano
%dir %{_datadir}/nano
%{_datadir}/nano/Dockerfile.nanorc

%changelog
* Mon Apr 11 2022 Maxwell G <gotmax@e.email> - 20.10.14-1
- Update to 20.10.14. Fixes rhbz#2063052.
- Mitigate CVE-2022-24769.

* Mon Jan 31 2022 Maxwell G <gotmax@e.email> - 20.10.12-3
- Fixes FTBFS. Closes rhbz#2046748.
- Use %%gobuild instead of Makefile to build binaries
- Add explanatory comments.
- Normalize install commands
- Make compliant with SourceURL Guidelines
- Remove no longer necessary `ExcludeArch: ppc64`.

* Thu Jan 20 2022 Fedora Release Engineering <releng@fedoraproject.org> - 20.10.12-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_36_Mass_Rebuild

* Tue Jan 11 2022 Maxwell G <gotmax@e.email> - 20.10.12-1
- Update to 20.10.12. Fixes rhbz#2032534.
- Install zsh completions to the correct directory. Fixes rhbz#2038888.

* Mon Nov 22 2021 Olivier Lemasle <o.lemasle@gmail.com> - 20.10-11-1
- Update to upstream 20.10.11 (fixes rhbz#2024384)
- Mitigates CVE-2021-41190 (fixes rhbz#2024940)

* Fri Oct 29 2021 Maxwell G <gotmax@e.email> - 20.10.10-1
- Update to 20.10.10 (fixes rhbz#2015385)
- Update virtual provides

* Fri Oct 08 2021 Maxwell G <gotmax@e.email> - 20.10.9-1
- Update to 20.10.9 (fixes rhbz#2010508)
- Patch seccomp policy to fix clone3() issue (fixes rhbz#2011523 and rhbz#1988199)

* Sun Aug 15 2021 Olivier Lemasle <o.lemasle@gmail.com> - 20.10.8-1
- Update to upstream 20.10.8 (fixes rhbz#1990148)
- Fix seccomp support (fixes rhbz#1986092)

* Sun Aug 15 2021 Dusty Mabe <dusty@dustymabe.com> - 20.10.7-3
- Remove `Requires(post)` on firewalld-filesystem.

* Thu Jul 22 2021 Fedora Release Engineering <releng@fedoraproject.org> - 20.10.7-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_35_Mass_Rebuild

* Fri Jul 16 2021 Olivier Lemasle <o.lemasle@gmail.com> - 20.10.7-1
- Update to upstream 20.10.7 (fixes rhbz#1967390)

* Tue May 04 2021 Olivier Lemasle <o.lemasle@gmail.com> - 20.10.6-2
- Add conflict with podman-docker

* Tue Apr 20 2021 Olivier Lemasle <o.lemasle@gmail.com> - 20.10.6-1
- Update to upstream 20.10.6 (#1948605)
- Re-bundle moby dependencies to fix gRPC issues with Swarm
  (https://github.com/coreos/fedora-coreos-tracker/issues/793)

* Sun Mar 14 2021 Olivier Lemasle <o.lemasle@gmail.com> - 20.10.5-1
- Update to latest upstream 20.10.5 - fixes #1903426
- Upstream brings compatibility with cgroups v2 - fixes #1746355
- Remove package moby-engine-vim (dockerfile.vim has been merged in upstream vim)
- Remove firewalld docker zone, since dockerd can now communicate with firewalld - fixes #1852680
- Build dockerd and docker-proxy from unbundled source packages
- Remove fixed storage-driver (cf. https://src.fedoraproject.org/rpms/moby-engine/pull-request/6)

* Tue Mar 02 2021 Zbigniew Jędrzejewski-Szmek <zbyszek@in.waw.pl> - 19.03.13-3.ce.git4484c46
- Rebuilt for updated systemd-rpm-macros
  See https://pagure.io/fesco/issue/2583.

* Tue Jan 26 2021 Fedora Release Engineering <releng@fedoraproject.org> - 19.03.13-2.ce.git4484c46
- Rebuilt for https://fedoraproject.org/wiki/Fedora_34_Mass_Rebuild

* Fri Oct 02 2020 Olivier Lemasle <o.lemasle@gmail.com> - 19.03.13-1.ce.git4484c46
- Update to upstream 19.03.13 (#1837641)

* Fri Oct 02 2020 Olivier Lemasle <o.lemasle@gmail.com> - 19.03.11-4.ce.git42e35e6
- Fix FTBFS: adapt to change to CMake builds (#1864160)

* Sat Aug 01 2020 Fedora Release Engineering <releng@fedoraproject.org> - 19.03.11-3.ce.git42e35e6
- Second attempt - Rebuilt for
  https://fedoraproject.org/wiki/Fedora_33_Mass_Rebuild

* Tue Jul 28 2020 Fedora Release Engineering <releng@fedoraproject.org> - 19.03.11-2.ce.git42e35e6
- Rebuilt for https://fedoraproject.org/wiki/Fedora_33_Mass_Rebuild

* Sun Jun 07 2020 Olivier Lemasle <o.lemasle@gmail.com> - 19.03.11-1.ce.git42e35e6
- Update to upstream 19.03.11 to prevent CVE-2020-13401

* Thu May 07 2020 Olivier Lemasle <o.lemasle@gmail.com> - 19.03.8-2.ce.gitafacb8b
- Configure storage-driver explicitely (fixes #1832301)
- Add firewalld zone: trust interface docker0, as firewalld now uses nftables
  by default and docker communicates with iptables (fixes #1817022)

* Mon Mar 16 2020 Olivier Lemasle <o.lemasle@gmail.com> - 19.03.8-1.ce.gitafacb8b
- Update to latest upstream release - Docker CE 19.03.8
- Prune unused BuildRequires

* Sun Mar 8 2020 Olivier Lemasle <o.lemasle@gmail.com> - 19.03.7-2.ce.git7141c19
- Add Conflicts with docker-ce-cli and Obsoletes docker-common

* Sat Mar 7 2020 Olivier Lemasle <o.lemasle@gmail.com> - 19.03.7-1.ce.git7141c19
- Update to latest upstream release - Docker CE 19.03.7
- Add Epoch: 2 to Obsoletes for docker and docker-latest

* Wed Jan 29 2020 Fedora Release Engineering <releng@fedoraproject.org> - 18.09.8-3.ce.git0dd43dd
- Rebuilt for https://fedoraproject.org/wiki/Fedora_32_Mass_Rebuild

* Thu Jul 25 2019 Fedora Release Engineering <releng@fedoraproject.org> - 18.09.8-2.ce.git0dd43dd
- Rebuilt for https://fedoraproject.org/wiki/Fedora_31_Mass_Rebuild

* Thu Jul 18 2019 Olivier Lemasle <o.lemasle@gmail.com> - 18.09.8-1.ce.git0dd43dd
- Update to latest upstream release - Docker CE 18.09.8

* Sat Jul 13 2019 Olivier Lemasle <o.lemasle@gmail.com> - 18.09.7-5.ce.git2d0083d
- Move docker-init and docker-proxy to /usr/libexec/docker
- Update moby-engine-nano summary to follow guidelines

* Sat Jul 13 2019 Olivier Lemasle <o.lemasle@gmail.com> - 18.09.7-4.ce.git2d0083d
- Add nofile ulimit to default docker daemon options (#1715254, #1708115)

* Fri Jul 12 2019 Olivier Lemasle <o.lemasle@gmail.com> - 18.09.7-3.ce.git2d0083d
- rebuilt

* Fri Jul 12 2019 Olivier Lemasle <o.lemasle@gmail.com> - 18.09.7-2.ce.git2d0083d
- Depend on packaged versions "runc" and "containerd" instead of building them.

* Thu Jun 27 2019 David Michael <dm0@redhat.com> - 18.09.7-1.ce.git2d0083d
- Update docker-ce to commit 2d0083d (version 18.09.7).
- Update runc to commit 425e105.
- Update containerd to commit 894b81a (1.2.6).
- Update docker-proxy to commit e7933d4.

* Tue May 14 2019 David Michael <dm0@redhat.com> - 18.09.6-1.ce.git481bc77
- Update docker-ce to commit 481bc77 (version 18.09.6).
- Update docker-proxy to commit 872f0a8.
- Obsolete and provide the docker and docker-latest packages. (#1700006)

* Thu Apr 11 2019 David Michael <dm0@redhat.com> - 18.09.5-1.ce.gite8ff056
- Update docker-ce to commit e8ff056 (version 18.09.5).
- Update docker-runc to commit 2b18fe1.
- Update docker-containerd to commit bb71b10 (version 1.2.5).
- Update docker-proxy to commit 4725f21.
- Report the correct engine version.
- Install symlinks to unprefixed runc/containerd program names.

* Thu Mar 28 2019 David Michael <dm0@redhat.com> - 18.06.3-2.ce.gitd7080c1
- Conflict with docker-common. (#1693397)

* Thu Feb 21 2019 David Michael <dm0@redhat.com> - 18.06.3-1.ce.gitd7080c1
- Update docker-ce to commit d7080c1 (version 18.06.3).

* Tue Feb 12 2019 David Michael <dm0@redhat.com> - 18.06.2-1.ce.git6d37f41
- Update docker-ce to commit 6d37f41 (version 18.06.2).
- Update docker-runc to commit a592beb.

* Mon Feb 11 2019 David Michael <dm0@redhat.com> - 18.06.1-3.ce.gite68fc7a
- Apply a runc patch for CVE-2019-5736.

* Fri Feb 01 2019 Fedora Release Engineering <releng@fedoraproject.org> - 18.06.1-2.ce.gite68fc7a
- Rebuilt for https://fedoraproject.org/wiki/Fedora_30_Mass_Rebuild

* Thu Nov 29 2018 David Michael <dm0@redhat.com> - 18.06.1-1.ce.gite68fc7a
- Update docker-ce to commit e68fc7a (version 18.06.1).
- Update docker-runc to commit 69663f0.
- Update docker-containerd to commit 468a545 (version 1.1.2).
- Update docker-proxy to commit 3ac297b.
- Backport a fix for mounting named volumes.
- Create a "docker" group for non-root Docker access.
- Support systemd socket-activation.
- Make runc and containerd commit IDs match their expected values.
- Preserve containerd debuginfo.

* Mon Nov 12 2018 Marcin Skarbek <rpm@skarbek.name> - 18.06.0-2.ce.git0ffa825
- add configuration file
- update service file

* Sat Aug 18 2018 Lokesh Mandvekar <lsm5@fedoraproject.org> - 18.06.0-1.ce.git0ffa825
- Resolves: #1539161 - first upload to Fedora
- built docker-ce commit 0ffa825
- built docker-runc commit ad0f5255
- built docker-containerd commit a88b631
- built docker-proxy commit a79d368
- built docker-init commit fec3683

* Tue Mar 20 2018 Lokesh Mandvekar <lsm5@fedoraproject.org> - 17.03.2-4.ce.gitf5ec1e2
- correct some rpmlint errors

* Wed Feb 21 2018 Lokesh Mandvekar <lsm5@fedoraproject.org> - 17.03.2-3.ce
- docker-* symlinks to moby-* (RE: gh PR 34226)

* Wed Feb 21 2018 Lokesh Mandvekar <lsm5@fedoraproject.org> - 17.03.2-2.ce
- rename binaries as per upstream gh PR 34226

* Fri Jan 26 2018 Lokesh Mandvekar <lsm5@fedoraproject.org> - 17.03.2-1
- initial build
- built moby commit f5ec1e2
- built cli commit 4b61f56
- built docker-runc commit 2d41c047
- built docker-containerd commit 3addd84
- built docker-proxy commit 7b2b1fe
